package com.qmth.eds.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.dto.ExamAssignDto;
import com.qmth.eds.bean.result.AssignResultPreviewResult;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.*;
import com.qmth.eds.common.enums.ExamAssignStatusEnum;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.ExamAssignMapper;
import com.qmth.eds.service.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 * 赋分管理 服务实现类
 * </p>
 */
@Service
public class ExamAssignServiceImpl extends ServiceImpl<ExamAssignMapper, ExamAssign> implements ExamAssignService {

    @Resource
    ExamSyncTotalService examSyncTotalService;

    @Resource
    ExamSyncStudentService examSyncStudentService;

    @Resource
    AsyncTaskService asyncTaskService;

    @Resource
    CloudMarkingScoreService cloudMarkingScoreService;

    @Resource
    CloudMarkingScoreForeignService cloudMarkingScoreForeignService;

    @Resource
    ExamAssignMapper examAssignMapper;

    @Override
    public IPage<ExamAssignDto> pageData(Long semesterId, Long examTypeId, Long collegeId, String courseCode, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<ExamAssignDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamAssignDto> assignListDtoIPage = this.baseMapper.pageData(page, collegeId, semesterId, examTypeId, courseCode);

        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }
        for (ExamAssignDto record : assignListDtoIPage.getRecords()) {
            int actualCount = 0;
            String kkxy = "";

            List<ExamSyncStudent> examSyncStudents = examSyncStudentService.listByExamSyncTotalId(examSyncTotal);
            if (!CollectionUtils.isEmpty(examSyncStudents)) {
                List<ExamSyncStudent> syncStudents = examSyncStudents.stream().filter(m -> m.getKch().equals(record.getCourseCode())).collect(Collectors.toList());
                actualCount = syncStudents.size();
                List<String> kkxyList = syncStudents.stream().map(ExamSyncStudent::getKkbm).distinct().collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(kkxyList) && kkxyList.size() > 1) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("课程[%s]查询出%d个开课学院，请联系管理员检查数据", courseCode, kkxyList.size()));
                }
                kkxy = kkxyList.get(0);
            }
            record.setActualCount(actualCount);
            record.setOpenCollege(kkxy);

            ExamAssign examAssign = this.baseMapper.getBySchoolIdAndSemesterIdAndExamTypeIdAndCourseCodeAndOpenCollege(collegeId, semesterId, examTypeId, record.getCourseCode(), kkxy);
            if (examAssign != null) {
                record.setId(examAssign.getId());
                record.setFormula(examAssign.getFormula());
                record.setStatus(examAssign.getStatus());
                record.setPublish(examAssign.getPublish());
            }
        }

        return assignListDtoIPage;
    }

    @Override
    public String calcData(String data) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        ExamAssign examAssign = JSON.parseObject(data, ExamAssign.class);
        // 保存赋分参数
        if (examAssign.getId() == null) {
            examAssign.setId(SystemConstant.getDbUuid());
            examAssign.setStatus(ExamAssignStatusEnum.INIT);
            examAssign.setCreateId(sysUser.getId());
            examAssign.setCreateTime(System.currentTimeMillis());
        } else {
            examAssign.setUpdateId(sysUser.getId());
            examAssign.setUpdateTime(System.currentTimeMillis());
        }
        this.saveOrUpdate(examAssign);

        // 计算赋分
        asyncTaskService.calcAssignScore(examAssign);
        return String.valueOf(examAssign.getId());
    }

    /**
     * 赋分结果预览
     *
     * @param iPage
     * @param semesterId
     * @param examTypeId
     * @param courseCode
     * @param openCollege
     * @param kcCollege
     * @param clazzId
     * @param teacherCode
     * @return
     */
    @Override
    public IPage<AssignResultPreviewResult> assignResultPreview(IPage<Map> iPage,
                                                                Long semesterId,
                                                                Long examTypeId,
                                                                Long collegeId,
                                                                String courseCode,
                                                                String openCollege,
                                                                String kcCollege,
                                                                String clazzId,
                                                                String teacherCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return examAssignMapper.assignResultPreview(iPage, schoolId, collegeId, semesterId, examTypeId, courseCode, openCollege, kcCollege, clazzId, teacherCode);
    }

    @Override
    public Map contrast(Long semesterId, Long examTypeId, String courseCode) {
        return null;
    }

    @Override
    public ExamAssignDto toCalc(Long semesterId, Long examTypeId, Long collegeId, String courseCode, String courseName, String openCollege) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        ExamAssign examAssign = this.baseMapper.getBySchoolIdAndSemesterIdAndExamTypeIdAndCourseCodeAndOpenCollege(collegeId, semesterId, examTypeId, courseCode, openCollege);
        ExamAssignDto examAssignDto = new ExamAssignDto();
        if (examAssign == null) {
            examAssign = new ExamAssign(collegeId, semesterId, examTypeId, courseCode, courseName, openCollege, null, null, ExamAssignStatusEnum.INIT, false);
            this.save(examAssign);
        }
        BeanUtils.copyProperties(examAssign, examAssignDto);

        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }
        int actualCount = 0;
        List<String> kcxyList = new ArrayList<>();
        List<ExamSyncStudent> examSyncStudents = examSyncStudentService.listByExamSyncTotalId(examSyncTotal);
        if (!CollectionUtils.isEmpty(examSyncStudents)) {
            List<ExamSyncStudent> syncStudents = examSyncStudents.stream().filter(m -> m.getKch().equals(courseCode) && m.getKkbm().equals(openCollege)).collect(Collectors.toList());
            actualCount = syncStudents.size();
            kcxyList = syncStudents.stream().map(ExamSyncStudent::getJgmc).distinct().collect(Collectors.toList());
        }
        examAssignDto.setActualCount(actualCount);
        examAssignDto.setInspectCollege(kcxyList);
        return examAssignDto;
    }

    @Transactional
    @Override
    public boolean publish(Long id, Long semesterId, Long examTypeId, Long collegeId, String courseCode, String courseName, String openCollege) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        ExamAssign examAssign;
        if (id == null) {
            examAssign = new ExamAssign(collegeId, semesterId, examTypeId, courseCode, courseName, openCollege, null, null, ExamAssignStatusEnum.INIT, false);
            examAssign.insertInfo(sysUser.getId());
            this.save(examAssign);
        } else {
            examAssign = this.getById(id);
            if(examAssign.getFormula() != null && !ExamAssignStatusEnum.FINISH.equals(examAssign.getStatus())){
                throw ExceptionResultEnum.ERROR.exception("赋分计算未完成，暂无法发布成绩");
            }
        }

        // 查询对外数据
        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }

        QueryWrapper<CloudMarkingScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CloudMarkingScore::getSchoolId, collegeId)
                .eq(CloudMarkingScore::getSemesterId, semesterId)
                .eq(CloudMarkingScore::getExamTypeId, examTypeId)
                .eq(CloudMarkingScore::getSubjectCode, courseCode);
        List<CloudMarkingScore> cloudMarkingScoreList = cloudMarkingScoreService.list(queryWrapper);

        List<CloudMarkingScoreForeign> cloudMarkingScoreForeigns = new ArrayList<>();
        List<ExamSyncStudent> examSyncStudents = examSyncStudentService.listByExamSyncTotalId(examSyncTotal);
        for (CloudMarkingScore cloudMarkingScore : cloudMarkingScoreList) {
            if (cloudMarkingScore.getExamSyncStudentId() != null) {
                ExamSyncStudent examSyncStudent = examSyncStudents.stream().filter(m -> cloudMarkingScore.getExamSyncStudentId().equals(m.getId())).findFirst().get();
                cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, cloudMarkingScore.getExamId(), examSyncStudent.getXnm(), examSyncStudent.getXqm(), examSyncStudent.getJxbId(), examSyncStudent.getKch(), examSyncStudent.getKcmc(), examSyncStudent.getXh(), cloudMarkingScore.getAssignScore()));
            } else {
                Optional<ExamSyncStudent> optional = examSyncStudents.stream()
                        .filter(s -> s.getJxbmc().equals(cloudMarkingScore.getClassName())
                                && ((StringUtils.isBlank(s.getCloudMarkingCourseCode()) && s.getKch().equals(cloudMarkingScore.getSubjectCode())) || (StringUtils.isNotBlank(s.getCloudMarkingCourseCode()) && s.getCloudMarkingCourseCode().equals(cloudMarkingScore.getSubjectCode())))
                                && s.getXh().equals(cloudMarkingScore.getStudentCode())).findFirst();
                if (!optional.isPresent()) {
                    cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, cloudMarkingScore.getExamId(), null, null, null, cloudMarkingScore.getSubjectCode(), cloudMarkingScore.getSubjectName(), cloudMarkingScore.getStudentCode(), cloudMarkingScore.getAssignScore()));
                } else {
                    ExamSyncStudent examSyncStudent = optional.get();
                    // 匹配上，保存考务数据表ID
                    cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, cloudMarkingScore.getExamId(), examSyncStudent.getXnm(), examSyncStudent.getXqm(), examSyncStudent.getJxbId(), examSyncStudent.getKch(), examSyncStudent.getKcmc(), examSyncStudent.getXh(), cloudMarkingScore.getAssignScore()));
                }
            }
        }

        List<Integer> examIds = cloudMarkingScoreList.stream().map(CloudMarkingScore::getExamId).distinct().collect(Collectors.toList());
        // 删除全量数据
        UpdateWrapper<CloudMarkingScoreForeign> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(CloudMarkingScoreForeign::getSchoolId, collegeId)
                .eq(CloudMarkingScoreForeign::getSemesterId, semesterId)
                .eq(CloudMarkingScoreForeign::getExamTypeId, examTypeId)
                .in(CloudMarkingScoreForeign::getExamId, examIds);
        cloudMarkingScoreForeignService.remove(updateWrapper);

        List<CloudMarkingScoreForeign> foreignData = new ArrayList<>();
        //1000条提交一次
        for (CloudMarkingScoreForeign scoreForeign : cloudMarkingScoreForeigns) {
            if (foreignData.size() == 1000) {
                cloudMarkingScoreForeignService.saveBatch(foreignData);
                foreignData.clear();
            }
            foreignData.add(scoreForeign);
        }
        //将剩下的数据也导入
        if (!foreignData.isEmpty()) {
            cloudMarkingScoreForeignService.saveBatch(foreignData);
        }

        // 更新发布状态为已发布
        UpdateWrapper<ExamAssign> examAssignUpdateWrapper = new UpdateWrapper<>();
        examAssignUpdateWrapper.lambda().set(ExamAssign::getPublish, true)
                .eq(ExamAssign::getId, examAssign.getId());
        return this.update(examAssignUpdateWrapper);
    }
}
