package com.qmth.eds.service.impl;

import com.qmth.boot.core.enums.Platform;
import com.qmth.eds.bean.auth.AuthBean;
import com.qmth.eds.bean.auth.ExpireTimeBean;
import com.qmth.eds.bean.result.LoginResult;
import com.qmth.eds.bean.result.UserLoginCheckResult;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.BasicSchool;
import com.qmth.eds.common.entity.SysRole;
import com.qmth.eds.common.entity.SysUser;
import com.qmth.eds.common.entity.TBSession;
import com.qmth.eds.common.enums.AppSourceEnum;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.AuthUtil;
import com.qmth.eds.common.util.RedisUtil;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.common.util.SessionUtil;
import com.qmth.eds.service.CommonCacheService;
import com.qmth.eds.service.SysRoleService;
import com.qmth.eds.service.TBSessionService;
import com.qmth.eds.service.TeachcloudCommonService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.security.NoSuchAlgorithmException;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 公共服务service impl
 */
@Service
public class TeachcloudCommonServiceImpl implements TeachcloudCommonService {

    @Resource
    CommonCacheService commonCacheService;


    @Resource
    TBSessionService tbSessionService;

    @Resource
    SysRoleService sysRoleService;

    @Resource
    RedisUtil redisUtil;

    /**
     * 登录公用
     *
     * @param password  密码
     * @param sysUser   用户对象
     * @param appSource 来源
     */
    @Override
    public LoginResult login(String password, SysUser sysUser, AppSourceEnum appSource) throws NoSuchAlgorithmException {
        Platform platform = ServletUtil.getRequestPlatform();
        String deviceId = ServletUtil.getRequestDeviceId();
        //生成token
        String token = SystemConstant.getUuid();
        commonCacheService.userCache(sysUser.getId());
        //添加用户会话缓存
        Set<String> roleNames = new HashSet<>();
        List<SysRole> sysRoles = sysRoleService.listByUserId(sysUser.getId());
        if (!sysRoles.isEmpty()) {
            roleNames = sysRoles.stream().map(m -> m.getType().name()).collect(Collectors.toSet());
        }
        String sessionId = SessionUtil.digest(sysUser.getId(), Math.abs(roleNames.toString().hashCode()), platform.name());
//      String test = SignatureEntityTest.build(SignatureType.TOKEN, sessionId, token);
        ExpireTimeBean expireTime = AuthUtil.getExpireTime(platform);
        TBSession tbSession = new TBSession(sysUser.getSchoolId(), sessionId, String.valueOf(sysUser.getId()), roleNames.toString(),
                platform.name(), platform.name(), deviceId, ServletUtil.getRequest().getLocalAddr(), token,
                expireTime.getDate().getTime(), appSource);
        tbSessionService.saveOrUpdate(tbSession);
        redisUtil.setUserSession(sessionId, tbSession, expireTime.getExpireSeconds());

        LoginResult loginResult = new LoginResult(sysUser, sessionId, token, roleNames, appSource);
        if (sysUser.getSchoolId() != null) {
            BasicSchool basicSchool = commonCacheService.schoolCache(sysUser.getSchoolId());
            loginResult.setSchoolInfo(Objects.nonNull(basicSchool) ? loginResult.new SchoolNativeBean(basicSchool) : null);
        }
        loginResult.setTime(System.currentTimeMillis());
        String mobileNumber = sysUser.getMobileNumber();
        int pwdCount = sysUser.getPwdCount();
        loginResult.setUserLoginCheckResult(new UserLoginCheckResult(sysUser.getId(), mobileNumber, pwdCount));
        if (sysUser.getOrgId() != null) {
            loginResult.setOrgId(String.valueOf(sysUser.getOrgId()));
        }
        return loginResult;
    }

    /**
     * 删除用户信息
     *
     * @param userId 用户ID
     * @param all    all
     */
    @Override
    public void removeUserInfo(Long userId, boolean all) throws NoSuchAlgorithmException {
        AuthBean authBean = commonCacheService.userAuthCache(userId);
        if (Objects.isNull(authBean)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        if (all) {
            for (Platform p : Platform.values()) {
                Set<String> roleNames = authBean.getRoleList().stream().map(SysRole::getName).collect(Collectors.toSet());
                String sessionId = SessionUtil.digest(userId, Math.abs(roleNames.toString().hashCode()), p.name());
                tbSessionService.removeById(sessionId);
                redisUtil.deleteUserSession(sessionId);
            }
        } else {
            TBSession tbSession = (TBSession) ServletUtil.getRequestSession();
            tbSessionService.removeById(tbSession.getId());
            redisUtil.deleteUserSession(tbSession.getId());
        }
        commonCacheService.removeUserCache(userId);
        commonCacheService.removeUserAuthCache(userId);
        commonCacheService.removeUserMenuCache(userId);
        commonCacheService.removeUserRolePrivilegeCache(userId);
    }

}
